SliceCommander = LibStub("AceAddon-3.0"):NewAddon("SliceCommander", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SliceCommander")

function SliceCommander:BarTexture()
	if SC.SET.BarTexture then
		return SliceCommander_BarTextures[SC.SET.BarTexture]
	else
		return "Interface\\AddOns\\SliceCommander\\Images\\Smooth.tga"
	end

end

function SliceCommander:SoundTest(name)
	if SliceCommander_Sounds[name] and SliceCommander_Sounds[name] ~= "" and SliceCommander_Sounds[name] ~= nil then
		local _
		if SC.previousSound then
			StopSound(SC.previousSound)
		end
		_, SC.previousSound = PlaySoundFile(SliceCommander_Sounds[name], SC.SET.soundChannel)
	end
end

function SliceCommander:PlaySound(name)
	if SliceCommander:isCombat() and SC.SET.SpellSound then
		if SliceCommander_Sounds[name] and SliceCommander_Sounds[name] ~= "" and SliceCommander_Sounds[name] ~= nil then
			local _
			if SC.previousSound then
				StopSound(SC.previousSound)
			end
			_, SC.previousSound = PlaySoundFile(SliceCommander_Sounds[name], SC.SET.soundChannel)
		end
	end
end

function SliceCommander:Sound(saved)
	if SliceCommander:isCombat() then
		if SC.SET[saved] then
			if SliceCommander_Sounds[SC.SET[saved]] ~= "" and SliceCommander_Sounds[SC.SET[saved]] ~= nil then
				PlaySoundFile(SliceCommander_Sounds[SC.SET[saved]], SC.SET.soundChannel )
			end
		end
	end
end

function SliceCommander:ChangeAnchor()
	local offSetSize, numberPos, numberNeg, alignPos, alignNeg
	offSetSize = SC.SET.BarMargin+SC.SET.TimerHeight
	numberPos = 0
	numberNeg = 0

	for i = 1, 20 do
		for j,v in pairs(SC.BARS) do
			if v['obj']:IsVisible() and v['expire'] == 0 then
				v['obj']:Hide()
			end
			if SC.SET.Pos[j] == i and v['expire'] ~= 0 then
				alignPos = -1*numberNeg*offSetSize-SC.SET.BarMargin
				v['obj']:ClearAllPoints()
				v['obj']:SCPoint("TOPRIGHT", VTimerEnergy, "BOTTOMRIGHT", 0, alignPos)
				numberNeg = numberNeg +1
			end
			if SC.SET.Pos[j] == i*-1 and v['expire'] ~= 0 then
				alignNeg = numberPos*offSetSize+SC.SET.BarMargin
				v['obj']:ClearAllPoints()
				v['obj']:SCPoint("BOTTOMRIGHT", VTimerEnergy, "TOPRIGHT", 0, alignNeg)
				numberPos = numberPos +1
			end
		end
	end
end

function SliceCommander:isCombat()
	if SC.SET.SoundCombat==true then
		return UnitAffectingCombat("player")
	else
		return true
	end
end

function SliceCommander:SetComboPts()
	local points = UnitPower("player", SPELL_POWER_COMBO_POINTS)
	local maxCP = UnitPowerMax("player", SPELL_POWER_COMBO_POINTS)
	
	for i = 1, maxCP do
		SC.BARS['CP']['obj'].combos[i]:Hide()
	end
	SC.BARS['CP']['obj']:Hide()
	local text = points
	if text == 0 then
		text = ''
	end
	
	if points < SC.BARS['CP']['expire'] then
		SC.BARS['CP']['last'] = SC.BARS['CP']['expire']
	end
	SC.BARS['CP']['expire'] = points
	
	SliceCommander_Combo:SetText(text)
	SC.BARS['CP']['obj'].comboText:SetText(text)
	
	if SC.SET.display.CP == true then
		for i = 1, points do
			SC.BARS['CP']['obj'].combos[i]:Show()
		end
		SC.BARS['CP']['obj']:Show()
	end
end

function SliceCommander:OnDisable()
	SliceCommander:UnregisterAllEvents()
end

function SliceCommander:AbilityCDBar(spell)
	local BAR = SC.BARS[spell]
	local display = SC.SET.display[spell]
	
	if display ~= true then
		BAR['expire'] = 0
		if BAR['obj']:IsVisible() then
			SliceCommander:HideBar(BAR)
		end
		return
	end

	local start, duration = GetSpellCooldown(BAR['name'])
	if start == nil or duration == nil then--not in this spec
		BAR['expire'] = 0
		if BAR['obj']:IsVisible() then--if you respect when a cd is enabled
			SliceCommander:HideBar(BAR)
		end
		return
	end
	if start ~= 0 and duration >= 2 and not BAR['obj']:IsVisible() then
		SliceCommander:EnableBar(BAR, duration, BAR['name'].." (CD)", (start+duration))
		return
	end
	if start == 0 then
		BAR['expire'] = 0
		if BAR['obj']:IsVisible() then
			SliceCommander:HideBar(BAR)
		end
		return
	end
	if start == 0 or duration < 2 then
		BAR['expire'] = 0
		return
	end
	BAR['expire'] = (start+duration)
	local x
	if BAR['expire'] > 0 and SC.tNow < BAR['expire'] then
		x = BAR['expire'] - SC.tNow
	else
		x = 0
	end
	BAR['obj']:SetValue(x)
	BAR['obj'].text:SetText(string.format("%0.1f", x))
end

function SliceCommander:ChargeAbilityCDBar(spell)
	local BAR = SC.BARS[spell]
	local display = SC.SET.display[spell]
	
	if display ~= true then
		BAR['expire'] = 0
		if BAR['obj']:IsVisible() then
			SliceCommander:HideBar(BAR)
		end
		return
	end

	local charges, maxCharges, start, duration  = GetSpellCharges(BAR['name'])
	if start == nil or duration == nil then--not in this spec
		BAR['expire'] = 0
		if BAR['obj']:IsVisible() then--if you respect when a cd is enabled
			SliceCommander:HideBar(BAR)
		end
		return
	end
	if charges ~= maxCharges and not BAR['obj']:IsVisible() then
		SliceCommander:EnableBar(BAR, duration, BAR['name'].." (CD)", (start+duration))
		return
	end
	if charges == maxCharges then
		BAR['expire'] = 0
		if BAR['obj']:IsVisible() then
			SliceCommander:HideBar(BAR)
		end
		return
	end
	
	BAR['expire'] = (start+duration)
	local x
	if BAR['expire'] > 0 and SC.tNow < BAR['expire'] then
		x = BAR['expire'] - SC.tNow
	else
		x = 0
	end
	
	local textcount = ''
	if charges > 0 then
		textcount = ' x'..charges
	end
	if not BAR['count'] or BAR['count'] ~= charges then
		if SC.SET.SpellText == true then
			BAR['obj'].text2:SetText(string.format("%s", BAR['name'].." (CD)"..textcount))
		else
			BAR['obj'].text2:SetText(textcount)
		end
	end
	
	BAR['obj']:SetValue(x)
	BAR['obj'].text:SetText(string.format("%0.1f", x))
end

function SliceCommander:EnableBar(BAR, duration, text, expire)
	BAR['expire'] = expire
	if SC.SET.FullTimer == true and duration then
		BAR['obj']:SetMinMaxValues(0, duration)
	else
		BAR['obj']:SetMinMaxValues(0, 6.0)
	end
	if SC.SET.SpellText == true then
		BAR['obj'].text2:SetText(string.format("%s", text))
	else
		BAR['obj'].text2:SetText('')
	end
	BAR['obj']:Show()
	SliceCommander:ChangeAnchor()
end

function SliceCommander:HideBar(BAR)
	BAR['expire'] = 0
	BAR['obj']:Hide()
	BAR['obj'].text:SetText("")
	SliceCommander:ChangeAnchor()
end

function SliceCommander:UpdateBar(spell)
	local BAR = SC.BARS[spell]
	local expire, icon, count, duration, unitCaster, x, _, maxValue, widthSeconde

	if SC.SET.display[spell] ~= true then
		BAR['expire'] = 0
		if BAR['obj']:IsVisible() then
			SliceCommander:HideBar(BAR)
		end
		return
	end

	local last = BAR['expire']
	if BAR['type'] == 1 then
		_, _, icon, count, _, duration, BAR['expire'], unitCaster = UnitAura("player", BAR['name'], nil, "PLAYER")
	elseif BAR['type'] == 2 then
		_, _, icon, count, _, duration, BAR['expire'], unitCaster = UnitDebuff("target", BAR['name'], nil, "PLAYER")
	elseif BAR['type'] == 3 then
		_, _, icon, count, _, duration, BAR['expire'], unitCaster = UnitDebuff("player", BAR['name'], nil, "PLAYER")
	else
		_, _, icon, count, _, duration, BAR['expire'], unitCaster = UnitDebuff("target", BAR['name'])
	end

	if unitCaster ~= "player" and BAR['type'] ~= 3 and unitCaster ~= nil then
		BAR['expire'] = last
	end

	if BAR['expire'] == nil then
		BAR['expire'] = 0
	end
	if SC.tNow == nil then
		SC.tNow = GetTime()
	end
	if BAR['expire'] > 0 and SC.tNow < BAR['expire'] then
		x = BAR['expire'] - SC.tNow
	else
		x = 0
	end
	if x > 0 then
		local xlast = SliceCommander:ShowBar(BAR, duration, icon)
		
		local textcount = ''
		if count and count > 1 then
			textcount = ' x'..count
		end
		if not BAR['count'] or BAR['count'] ~= count then
			if SC.SET.SpellText == true then
				BAR['obj'].text2:SetText(string.format("%s", BAR['name']..textcount))
			else
				BAR['obj'].text2:SetText(textcount)
			end
		end
		BAR['count'] = count
		BAR['obj']:SetValue(x)
		BAR['obj'].text:SetText(string.format("%0.1f", x))

		SliceCommander:Tick(BAR, x)
		SliceCommander:Pandemic(BAR, x)
		SliceCommander:SoundBar(BAR, x, xlast)
	else
		if BAR['obj']:IsVisible() then
			if BAR['sound'] ~= nil then
				SliceCommander:PlaySound(BAR['sound']['expired'])
			end
			SliceCommander:HideBar(BAR)
		end
	end
end

function SliceCommander:CreateTOTMacro()
	if not UnitAffectingCombat("player") then
		local alreadyExist = GetMacroBody("SC_TOT")
		if alreadyExist then
			EditMacro("SC_TOT", "SC_TOT", "INV_MISC_QUESTIONMARK", "/cast [@"..SC.SET.TOTTarget..",modifier:alt] [help] [@targettarget, help][@none,@="..SC.SET.TOTTarget.."] "..SC_SPELL['TOT'], 1, 1)
		else
			CreateMacro("SC_TOT", "INV_MISC_QUESTIONMARK", "/cast [@"..SC.SET.TOTTarget..",modifier:alt] [help] [@targettarget, help][@none,@="..SC.SET.TOTTarget.."] "..SC_SPELL['TOT'], 1, 1)
		end
	end
end

function SliceCommander:HealthBar(bar)
	if SC.SET.display[bar] then
		local HP, maxHP, pourcentHP, hexCol, rCol, gCol, bCol, healthText
		HP = UnitHealth(SC.BARS[bar]['target'])
		maxHP = UnitHealthMax(SC.BARS[bar]['target'])
		if maxHP ~= 0 then
			pourcentHP = (HP / maxHP) * 100
		else
			pourcentHP = 0
		end
		if maxHP == nil then
			maxHP = 0
		end
		if HP == nil then
			HP = 0
		end
		if HP == 0 then
			if SC.BARS[bar]['obj']:IsVisible() then
				SC.BARS[bar]['obj']:Hide()
			end
			SC.BARS[bar]['expire'] = 0
		else
			if SC.BARS[bar]['expire'] > 35 and pourcentHP < 35 then
				SliceCommander:PlaySound(SC.SET[bar].LowUnder)
			elseif SC.BARS[bar]['expire'] > 50 and pourcentHP < 50 then
				SliceCommander:PlaySound(SC.SET[bar].MidUnder)
			end
			hexCol = (100-pourcentHP)*(255/100)
			rCol = hexCol
			gCol = 255-hexCol
			if rCol > 210 then
				rCol = 210
			end
			if rCol < 50 then
				rCol = 50
			end
			if gCol > 210 then
				gCol = 210
			end
			if gCol < 50 then
				gCol = 50
			end
			bCol = 50

			if SC.SET[bar].HideCombat and not UnitAffectingCombat("player") then
				if SC.BARS[bar]['obj']:IsVisible() then
					SC.BARS[bar]['obj']:Hide()
				end
				SC.BARS[bar]['expire'] = 0
			else
				if not SC.BARS[bar]['obj']:IsVisible() then
					SC.BARS[bar]['obj']:Show()
				end
				SC.BARS[bar]['expire'] = pourcentHP
			end
			SC.BARS[bar]['obj']:SetValue(pourcentHP)
			if SC.SET[bar].value then
				if HP > 1000000000 then
					healthText = string.format("%0.1f", HP/1000000000)..'G'
				elseif HP > 1000000 then
					healthText = string.format("%0.1f", HP/1000000)..'M'
				elseif HP > 1000 then
					healthText = string.format("%0.1f", HP/1000)..'K'
				else
					healthText = HP
				end
			else
				healthText = string.format("%0.1f", pourcentHP)
			end
			SC.BARS[bar]['obj'].text:SetText(healthText)
			SC.BARS[bar]['obj']:SetStatusBarColor(rCol/255,gCol/255,bCol/255)
		end
	else
		SC.BARS[bar]['expire'] = 0
		SC.BARS[bar]['obj']:Hide()
	end
end


function SliceCommander:TargetThreat()
    local hexCol, rCol, gCol, bCol
	if SC.SET.display.THREAT == false or (SC.SET.ThreatGroupOnly and IsInRaid() == false and IsInGroup() == false) then
		SC.BARS['THREAT']['expire'] = 0
	else
		local isTanking, status, threatpct, rawthreatpct, threatvalue = UnitDetailedThreatSituation("player", "target")
		if rawthreatpct == nil then
			SC.BARS['THREAT']['expire'] = 0
		else
			hexCol = rawthreatpct *(255/110)
			rCol = hexCol-45
			gCol = 255-hexCol+50
			if isTanking == 1 then
				rCol = 210
				gCol = 50
				rawthreatpct = 100
			end
			if rCol > 210 then
				rCol = 210
			end
			if rCol < 50 then
				rCol = 50
			end
			if gCol > 210 then
				gCol = 210
			end
			if gCol < 50 then
				gCol = 50
			end
			bCol = 50
			SC.BARS['THREAT']['expire'] = rawthreatpct
		end
	end
	if SC.BARS['THREAT']['expire'] == 0 then
		SC.BARS['THREAT']['obj']:Hide()
	else
		SC.BARS['THREAT']['obj']:SetValue(SC.BARS['THREAT']['expire'])
		SC.BARS['THREAT']['obj'].text:SetText(string.format("%0.1f", SC.BARS['THREAT']['expire']))
		SC.BARS['THREAT']['obj']:SetStatusBarColor(rCol/255,gCol/255,bCol/255)
		SC.BARS['THREAT']['obj']:Show()
		SliceCommander:ChangeAnchor()
	end
end

function SliceCommander:SetTransparency(value)
	VTimerEnergy:SetAlpha(value/100)
	SC.OTHER['AS']['obj']:SetAlpha(value/100)
end

function SliceCommander:TestBF()
	if SC.SET.display.BF then
		local name = UnitAura("player", SC_SPELL['BF'], nil, "PLAYER")
		if name and not SC.OTHER['BF']['obj']:IsVisible() then
			SC.OTHER['BF']['obj']:Show()
		elseif not name and SC.OTHER['BF']['obj']:IsVisible() then
			if SC.SET.IsLocked == true then
				SC.OTHER['BF']['obj']:Hide()
			end
		end
	else
		if SC.SET.IsLocked == true then
			SC.OTHER['BF']['obj']:Hide()
		end
	end
end



function SliceCommander:TestBB()
	if SC.SET.display.BB then
		local icon = GetSpellTextureFileName(SC_SPELL['PS'])
		if icon and icon == 'INTERFACE\\ICONS\\inv_weapon_rifle_07' and not SC.OTHER['BB']['obj']:IsVisible() then
			SC.OTHER['BB']['obj']:Show()
		elseif icon ~= 'INTERFACE\\ICONS\\inv_weapon_rifle_07' and SC.OTHER['BB']['obj']:IsVisible() then
			if SC.SET.IsLocked == true then
				SC.OTHER['BB']['obj']:Hide()
			end
		end
	else
		if SC.SET.IsLocked == true then
			SC.OTHER['BB']['obj']:Hide()
		end
	end
end

function SliceCommander:TestDP()
	local combat = UnitAffectingCombat("player")
	if SC.SET.display.DPICON and GetSpecialization() == 1 then 
		local mhPoison = SC.SET.poison.mh
		local _, _, _, selected = GetTalentInfo(6, 1, 1)
		if SC.SET.poison.mh == SC_SPELL['AP'] and not selected then
			mhPoison = SC_SPELL['DP']
		end
		
		local ohPoison = SC.SET.poison.oh
		local _, _, _, selected = GetTalentInfo(4, 1, 1)
		if not selected then
			ohPoison = SC_SPELL['CP']
		end
		
		local name, _, _, _, _, _, expirationTime = UnitAura("player", mhPoison, nil, "PLAYER")
		local name2, _, _, _, _, _, expirationTime2 = UnitAura("player", ohPoison, nil, "PLAYER")

		local minimalTime = SC.SET.display.DPTIME*60

		if IsMounted() or ((name and ((expirationTime-SC.tNow) > minimalTime or combat)) and (name2 and ((expirationTime2-SC.tNow) > minimalTime or combat))) then
			if SC.SET.IsLocked == true then
				if SC.OTHER['POISON']['obj']:GetAlpha() == 1 then
					SC.OTHER['POISON']['obj']:SetAlpha(0)
				end
				if not combat and SC.OTHER['POISON']['obj'].button:IsMouseEnabled() then
					SC.OTHER['POISON']['obj'].button:EnableMouse(false)
				end
			end
		elseif not name or not name2 or (((expirationTime-SC.tNow) < minimalTime or (expirationTime2-SC.tNow) < minimalTime) and not combat) and not IsMounted() then
			if SC.OTHER['POISON']['obj']:GetAlpha() ~= 1 then
				SliceCommander:SoundTest(SC.SET.DPSound)
				SC.OTHER['POISON']['obj']:SetAlpha(1)
			end
			if not combat and not SC.OTHER['POISON']['obj'].button:IsMouseEnabled() and SC.SET.display.DPBUT then
				SC.OTHER['POISON']['obj'].button:EnableMouse(true)
			end
		end
	else
		if SC.SET.IsLocked == true then
			if SC.OTHER['POISON']['obj']:GetAlpha() == 1 then
				SC.OTHER['POISON']['obj']:SetAlpha(0)
			end
			if not combat and SC.OTHER['POISON']['obj'].button:IsMouseEnabled() then
				SC.OTHER['POISON']['obj'].button:EnableMouse(false)
			end
		end
	end
end

function SliceCommander:TestSOTD()
	if SC.SET.display.SOTD then
		local name, _, _, count = UnitAura("player", SC_SPELL['SOTD'], nil, "PLAYER")
		if name and count >= 29 then
			if not SC.OTHER['SOTD']['obj']:IsVisible() then
				SC.OTHER['SOTD']['obj']:Show()
			end
			SC.OTHER['SOTD']['obj'].iconTxt:SetText(count)
		elseif SC.OTHER['SOTD']['obj']:IsVisible() then
			if SC.SET.IsLocked == true then
				SC.OTHER['SOTD']['obj']:Hide()
			end
			SC.OTHER['SOTD']['obj'].iconTxt:SetText('')
		end
	else
		if SC.SET.IsLocked == true then
			SC.OTHER['SOTD']['obj']:Hide()
		end
		SC.OTHER['SOTD']['obj'].iconTxt:SetText('')
	end
end

function SliceCommander:Latency()
	local _
	SC.tNow = GetTime()
	if (SC.SET.PadLatency) then
		 _, _, SC.lag = GetNetStats()
		SC.tNow = SC.tNow + (SC.lag*2/1000)
	end
end

function SliceCommander:Energy()
	VTimerEnergy:SetValue(UnitMana("player"))
	VTimerEnergy:SetMinMaxValues(0,UnitManaMax("player"))
	VTimerEnergyTxt:SetText(VTimerEnergy:GetValue())

	if UnitManaMax("player") == UnitMana('player') and not UnitAffectingCombat("player") then
		SliceCommander:SetTransparency(SC.SET.EnergyTrans)
	else
		SliceCommander:SetTransparency(SC.SET.FrameOpacity)
	end
end

function SliceCommander:GCD()
  	if not SC.OTHER['AS']['expire'] then return SC.OTHER['AS']['obj']:Hide() end

	local val = (SC.OTHER['AS']['expire']-GetTime())
	if val < 0 then
		SC.OTHER['AS']['expire'] = nil
	else
		SC.OTHER['AS']['obj']:SetValue(val)
	end
end

function SliceCommander:SoundBar(BAR, current, previous)
	if BAR['sound'] ~= nil then
		if previous >= 3 and current < 3 then
			SliceCommander:PlaySound(BAR['sound']['alert3'])
		elseif previous >= 2 and current < 2 then
			SliceCommander:PlaySound(BAR['sound']['alert2'])
		elseif previous >= 1 and current < 1 then
			SliceCommander:PlaySound(BAR['sound']['alert1'])
		end

		if previous > 0 and previous <= 1 and current > 1 then
			SliceCommander:PlaySound(BAR['sound']['refresh1'])
		elseif previous <= 2 and current > 2 then
			SliceCommander:PlaySound(BAR['sound']['refresh2'])
		elseif previous <= 3 and current > 3 then
			SliceCommander:PlaySound(BAR['sound']['refresh3'])
		end
	end
end

function SliceCommander:Pandemic(BAR, current)
	if SC.SET.Pandemic and BAR['pandemic'] then
		local timers = BAR['pandemictimer']
		if type(timers) ~= 'table' then
			local maxCP = UnitPowerMax("player", SPELL_POWER_COMBO_POINTS)
			timers = {}
			for i = 0, maxCP do
				timers[i] = BAR['pandemictimer']
			end
		end
		local points = UnitPower("player", SPELL_POWER_COMBO_POINTS)
		if BAR['pandemicobj']  == nil then
			BAR['pandemicobj'] = SliceCommander:NewTickArray(BAR)
			BAR['pandemicobj']:SetVertexColor(255/255, 0, 0, 1)
		end
		local _, maxValue = BAR['obj']:GetMinMaxValues()
		local widthSeconde = (BAR['obj']:GetWidth())/maxValue
		local value = current-(timers[points]*.3)
		if value >= 0 and value <= maxValue then
			BAR['pandemicobj']:Show()
			BAR['pandemicobj']:SetPoint("LEFT", BAR['obj'], "LEFT", widthSeconde*value, 0)
		else
			BAR['pandemicobj']:Hide()
		end
	end
end

function SliceCommander:Tick(BAR, current)
	if SC.SET.Tick and BAR['tick'] ~= nil and BAR['tick']['value'] ~= nil then
		local tick = BAR['tick']['value']
		local _, maxValue = BAR['obj']:GetMinMaxValues()
		local widthSeconde = (BAR['obj']:GetWidth())/maxValue
		local value = (current % tick)
		if value == 0 then
			value = tick
		end
		local j = maxValue/BAR['tick']['value'];
		for i = 1,j do
			if BAR['tick']['obj'][i] == nil then
				BAR['tick']['obj'][i] = SliceCommander:NewTickArray(BAR)--CREATE TICK OBJECT IF NECESSARY
			end
			if value > current or value > maxValue then
				BAR['tick']['obj'][i]:Hide()
			else
				BAR['tick']['obj'][i]:Show()
				BAR['tick']['obj'][i]:SetPoint("LEFT", BAR['obj'], "LEFT", widthSeconde*value, 0)
				value = value+tick
			end
		end
	end
end

function SliceCommander:ShowBar(BAR, duration, icon)
	if not BAR['obj']:IsVisible() then
		if BAR['sound'] ~= nil then
			SliceCommander:PlaySound(BAR['sound']['applied'])
		end
		if icon then
			BAR['obj'].icon:SetTexture(icon)
		end
		SliceCommander:EnableBar(BAR, duration, BAR['name'], BAR['expire'])

		return 0;
	else
		return BAR['obj']:GetValue()
	end
end

function SliceCommander:MultipleAbilityBar(spell)
	local BAR = SC.BARS[spell]
	local expire, duration, x, _, expire
	if SC.SET.display[spell] ~= true then
		BAR['expire'] = 0
		if BAR['obj']:IsVisible() then
			SliceCommander:HideBar(BAR)
		end
		return
	end

	local last = BAR['expire']
	local spellname = {}
	BAR['expire'] = 0
	for i, v in ipairs(BAR.subbar) do
		_, _, _, _, _, duration, expire = UnitAura("player", SC_SPELL[v], nil, "PLAYER")
		if expire ~= nil then
			if SC.SET.RB[v] then
				table.insert(spellname, SC.SET.RB[v])
			end
			BAR['expire'] = expire
		end
	end
	
	if SC.tNow == nil then
		SC.tNow = GetTime()
	end
	if BAR['expire'] > 0 and SC.tNow < BAR['expire'] then
		x = BAR['expire'] - SC.tNow
	else
		x = 0
	end
	if x > 0 then
		local xlast = SliceCommander:ShowBar(BAR, duration, false)

		if SC.SET.SpellText == true then
			local displayConcatName = spell..'Name'
			if SC.SET.display[displayConcatName] and next(spellname) ~= nil then
				BAR['obj'].text2:SetText(table.concat(spellname, " - "))
			else
				BAR['obj'].text2:SetText(BAR['name'])
			end
		end
		BAR['obj']:SetValue(x)
		BAR['obj'].text:SetText(string.format("%0.1f", x))

		SliceCommander:Pandemic(BAR, x)
		SliceCommander:SoundBar(BAR, x, xlast)
	else
		if BAR['obj']:IsVisible() then
			if BAR['sound'] ~= nil then
				SliceCommander:PlaySound(BAR['sound']['expired'])
			end
			SliceCommander:HideBar(BAR)
		end
	end
end

--MAIN
SLASH_RELOADUI1 = "/rl"
SlashCmdList.RELOADUI = ReloadUI

SLASH_SLICECOMMANDER1 = "/slicecommander"
SLASH_SLICECOMMANDER2 = "/sc"
SlashCmdList["SLICECOMMANDER"] = function(msg, editbox)
	local command, rest = msg:match("^(%S*)%s*(.-)$")
	if command == "lock" or command == "unlock" then
		SliceCommander:LockCkecked(not SC.SET.IsLocked)
	elseif command == "eyefinity" and tonumber(rest) then
		SC.SET.eyefinity = rest
		ReloadUI()
	elseif command == "tot" and tostring(rest) then
		SC.SET.TOTTarget = rest
		SliceCommander:CreateTOTMacro()
		print(L['Macro update'])
	else
		InterfaceOptionsFrame_OpenToCategory("SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
		InterfaceOptionsFrame_OpenToCategory("SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	end
end
